import { Message, Request, Response } from "./message";
import { Log } from "./log";
import { IVia } from "./_via";
export interface ContextTask {
    readonly name: string;
    readonly complete: Promise<void>;
}
export interface Context extends AsyncDisposable {
    id: string;
    connection: IVia<Context>;
    err?: any;
    in?: Message<any>;
    out?: Message<any>;
    req?: Request<any>;
    res?: Response<any>;
    params?: {
        [key: string]: string;
    };
    isReq(inbound?: boolean): this is RequestContext;
    isRes(inbound?: boolean): this is ResponseContext;
    /** sub tasks that run separately*/
    readonly tasks: ContextTask[];
    readonly complete: Promise<void>;
    onDispose(cb: () => void | Promise<void>): void;
    [key: string]: any;
}
export interface ContextConstructor<C extends Context = Context> {
    new (init: {
        connection: IVia<C>;
        in: Message<any>;
        log: Log;
    }): C;
    new (init: {
        connection: IVia<C>;
        out: Message<any>;
        log: Log;
    }): C;
}
export declare class DefaultContext implements Context {
    private _disposers;
    private _tasks;
    id: string;
    connection: IVia<Context>;
    in?: Message;
    out?: Message;
    constructor(init: {
        connection: IVia<Context>;
        in?: Message;
        out?: Message;
    });
    isReq(inbound?: boolean): this is RequestContext;
    isRes(inbound?: boolean): this is ResponseContext;
    private defaultResponse;
    send(msg: Message): void;
    get tasks(): ContextTask[];
    get complete(): Promise<void>;
    onDispose(cb: () => void | Promise<void>): void;
    [Symbol.asyncDispose](): Promise<void>;
}
export interface RequestContext extends ResponseContext {
    req: Request;
}
export interface ResponseContext extends Context {
    res: Response;
}
