"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultContext = void 0;
const core_1 = require("@viae/core");
class DefaultContext {
    _disposers = [];
    _tasks = [];
    id;
    connection;
    in;
    out;
    constructor(init) {
        Object.assign(this, init);
        if (this.isReq()) {
            this.out = this.defaultResponse(this.in);
        }
    }
    isReq(inbound = true) {
        return (inbound) ?
            this.in !== undefined && this.in.head.status === undefined :
            this.out !== undefined && this.out.head.status === undefined;
    }
    isRes(inbound = true) {
        return (inbound) ?
            this.in !== undefined && this.in.head.status !== undefined :
            this.out !== undefined && this.out.head.status !== undefined;
    }
    defaultResponse(req) {
        if (req) {
            return {
                id: req.id,
                head: { status: core_1.Status.NotFound }
            };
        }
    }
    send(msg) {
        msg.id = typeof msg.id === "string" ? msg.id : this.in.id;
        this.out = msg;
    }
    get tasks() {
        return this._tasks;
    }
    get complete() {
        return Promise.all(this._tasks.map(x => x.complete)).then(function () { void 0; });
    }
    onDispose(cb) {
        this._disposers.push(cb);
    }
    async [Symbol.asyncDispose]() {
        await Promise.all(this._disposers);
        this._disposers = [];
    }
}
exports.DefaultContext = DefaultContext;
