"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.All = All;
exports.Get = Get;
exports.Put = Put;
exports.Post = Post;
exports.Delete = Delete;
exports.Subscribe = Subscribe;
exports.Data = Data;
exports.Head = Head;
exports.Raw = Raw;
exports.Path = Path;
exports.Ctx = Ctx;
exports.Next = Next;
exports.Param = Param;
exports.Controller = Controller;
exports.Use = Use;
exports.Trace = Trace;
require("reflect-metadata");
const log_1 = require("./log");
function methodDecorator(method, path, opts) {
    return function (target, propertyKey, parameterDesc) {
        let router = Reflect.getMetadata("__router", target) || {};
        let routes = router.routes || [];
        let route = routes[propertyKey] || {};
        route.method = method;
        route.path = path;
        route.opts = opts || { end: true };
        routes[propertyKey] = route;
        router["routes"] = routes;
        Reflect.defineMetadata("__router", router, target);
    };
}
function All(path, opts) {
    return methodDecorator(undefined, path || "", opts);
}
function Get(path, opts) {
    return methodDecorator("GET", path || "", opts);
}
function Put(path, opts) {
    return methodDecorator("PUT", path || "", opts);
}
function Post(path, opts) {
    return methodDecorator("POST", path || "", opts);
}
function Delete(path, opts) {
    return methodDecorator("DELETE", path || "/", opts);
}
function Subscribe(path, opts) {
    return methodDecorator("SUBSCRIBE", path || "/", opts);
}
function Data() {
    return function (target, propertyKey, index) {
        let router = Reflect.getMetadata("__router", target) || {};
        let routes = router.routes || [];
        let route = routes[propertyKey] || {};
        route.args = route.args || [];
        route.args[index] = { type: "data" };
        routes[propertyKey] = route;
        router["routes"] = routes;
        Reflect.defineMetadata("__router", router, target);
    };
}
function Head() {
    return function (target, propertyKey, index) {
        let router = Reflect.getMetadata("__router", target) || {};
        let routes = router.routes || [];
        let route = routes[propertyKey] || {};
        route.args = route.args || [];
        route.args[index] = { type: "head" };
        routes[propertyKey] = route;
        router["routes"] = routes;
        Reflect.defineMetadata("__router", router, target);
    };
}
function Raw() {
    return function (target, propertyKey, index) {
        let router = Reflect.getMetadata("__router", target) || {};
        let routes = router.routes || [];
        let route = routes[propertyKey] || {};
        route.args = route.args || [];
        route.args[index] = { type: "raw" };
        routes[propertyKey] = route;
        router["routes"] = routes;
        Reflect.defineMetadata("__router", router, target);
    };
}
/** get the matched path for the route*/
function Path() {
    return function (target, propertyKey, index) {
        let router = Reflect.getMetadata("__router", target) || {};
        let routes = router.routes || [];
        let route = routes[propertyKey] || {};
        route.args = route.args || [];
        route.args[index] = { type: "path" };
        routes[propertyKey] = route;
        router["routes"] = routes;
        Reflect.defineMetadata("__router", router, target);
    };
}
function Ctx() {
    return function (target, propertyKey, index) {
        let router = Reflect.getMetadata("__router", target) || {};
        let routes = router.routes || [];
        let route = routes[propertyKey] || {};
        route.args = route.args || [];
        route.args[index] = { type: "ctx" };
        routes[propertyKey] = route;
        router["routes"] = routes;
        Reflect.defineMetadata("__router", router, target);
    };
}
function Next() {
    return function (target, propertyKey, index) {
        let router = Reflect.getMetadata("__router", target) || {};
        let routes = router.routes || [];
        let route = routes[propertyKey] || {};
        route.args = route.args || [];
        route.args[index] = { type: "next" };
        routes[propertyKey] = route;
        router["routes"] = routes;
        Reflect.defineMetadata("__router", router, target);
    };
}
function Param(key, type) {
    return function (target, propertyKey, index) {
        let router = Reflect.getMetadata("__router", target) || {};
        let routes = router.routes || [];
        let route = routes[propertyKey] || {};
        route.args = route.args || [];
        route.args[index] = {
            type: "param",
            opt: key,
            ctor: type || String
        };
        routes[propertyKey] = route;
        router["routes"] = routes;
        Reflect.defineMetadata("__router", router, target);
    };
}
function Controller(root) {
    return function (target) {
        let router = Reflect.getMetadata("__router", target.prototype) || {};
        router.root = root || "";
        Reflect.defineMetadata("__router", router, target.prototype);
    };
}
function Use(root) {
    return function (target, propertyKey) {
        let router = Reflect.getMetadata("__router", target) || {};
        let routes = router.routes || [];
        let route = routes[propertyKey] || {};
        route.controller = propertyKey;
        route.root = root || "";
        routes[propertyKey] = route;
        router["routes"] = routes;
        Reflect.defineMetadata("__router", router, target);
    };
}
const _nolog = new log_1.ConsoleLog();
function Trace() {
    return function (target, propertyKey, descriptor) {
        // keep a reference to the original function
        const originalValue = descriptor.value;
        // Replace the original function with a wrapper
        descriptor.value = function (...args) {
            const ctx = args[0] || {};
            const log = ctx.log || _nolog;
            log.debug(`=> ${propertyKey}(${args.join(", ")})`);
            // Call the original function
            var result = originalValue.apply(this, args);
            log.debug(`<= ${result}`);
            return result;
        };
    };
}
;
