"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Api = void 0;
const error_1 = require("../error");
const readable_stream_1 = require("./readable-stream");
const router_1 = require("./router");
class Api extends router_1.Router {
    constructor(root) {
        super({ root: root || "/" });
        const methodFn = (method) => (opts) => {
            const { path, handler } = opts;
            let isNext = (opts.next === true) ? true : false;
            let end = opts.end != undefined ? true : false;
            super.route({
                path,
                method,
                end,
                process: [
                    async function (ctx, next) {
                        const log = ctx.connection.log;
                        let args = {
                            data: ctx.in.data,
                            head: ctx.in.head,
                            raw: ctx.in.raw,
                            path: ctx.in.head.matchedPath,
                            ctx: ctx,
                            params: ctx.params
                        };
                        log.debug({
                            method: method,
                            end,
                            path: args.path,
                            params: args.params,
                            data: args.data,
                        }, "processing api route");
                        if (isNext) {
                            args.next = next;
                        }
                        try {
                            if (opts.accept) {
                                if (opts.accept == "stream") {
                                    if (!(0, readable_stream_1.isReadableStream)(args.data)) {
                                        log.debug({
                                            method: method,
                                            path: args.path,
                                        }, "failed accept check");
                                        throw new error_1.ViaeError(400, "invalid argument. expected stream");
                                    }
                                }
                                if (opts.accept == "object") {
                                    if (args.data === undefined || (0, readable_stream_1.isReadableStream)(args.data)) {
                                        log.debug({
                                            method: method,
                                            path: args.path,
                                        }, "failed accept check");
                                        throw new error_1.ViaeError(400, "invalid argument. expected object");
                                    }
                                }
                            }
                            if (opts.validate) {
                                if (!(0, readable_stream_1.isReadableStream)(args.data)) {
                                    try {
                                        opts.validate(args.data);
                                    }
                                    catch (err) {
                                        log.debug({
                                            method: method,
                                            end,
                                            path: args.path,
                                            params: args.params,
                                            data: args.data,
                                            err
                                        }, "failed validation");
                                        throw new error_1.ViaeError(400, err.message ? err.message : "unknown validation error");
                                    }
                                }
                                else {
                                    args.data = args.data.pipeThrough(new TransformStream({
                                        transform(chunk, controller) {
                                            try {
                                                opts.validate(chunk);
                                            }
                                            catch (err) {
                                                log.debug({
                                                    method: method,
                                                    end,
                                                    path: args.path,
                                                    params: args.params,
                                                    data: args.data,
                                                    err
                                                }, "failed validation");
                                                throw new error_1.ViaeError(400, err.message ? err.message : "unknown validation error");
                                            }
                                            controller.enqueue(chunk);
                                        }
                                    }), {});
                                }
                            }
                            const result = await handler(args);
                            log.debug({ result }, "handler returned");
                            if (result !== undefined) {
                                ctx.out.data = result;
                            }
                            if (isNext == false) {
                                ctx.out.head.status = 200;
                            }
                        }
                        catch (err) {
                            if (typeof err == "number") {
                                ctx.out.head.status = err;
                            }
                            else if (err instanceof error_1.ViaeError) {
                                ctx.out.head.status = err.status;
                                ctx.out.data = err.message;
                            }
                            else {
                                throw err;
                            }
                        }
                    }
                ]
            });
        };
        this.all = methodFn(null);
        this.get = methodFn("GET");
        this.post = methodFn("POST");
        this.put = methodFn("PUT");
        this.delete = methodFn("DELETE");
        this.subscribe = methodFn("SUBSCRIBE");
    }
    all;
    get;
    post;
    put;
    delete;
    subscribe;
    use(arg1, arg2) {
        if (typeof arg1 == "string") {
            let path = arg1;
            let api = arg2;
            super.route({ path, method: null, process: [api] });
        }
        else {
            super.use(arg1);
        }
    }
}
exports.Api = Api;
