import { Context } from '../context';
import { Processor, Middleware } from 'rowan';
import { Log } from '../log';
export type InterceptOptions<Ctx extends Context = Context> = {
    id: string;
    handlers: Processor<Ctx>[];
};
/**
 * used to intercept messages with matching message ids and terminate further processing
 **/
export default class Interceptor<Ctx extends Context = Context> implements Middleware<Ctx> {
    private _log;
    private _interceptors;
    meta: {
        type: "Interceptor";
    };
    constructor(_log: Log);
    /**
     * intercept any message with a matching id
     * @param id: message id to intercept
     * @param handlers: handlers or middleware
     * @returns method to remove (dispose) the interception entry.`
     */
    intercept(opt: InterceptOptions): () => void;
    /**
     * re-route processing to an interceptor's middleware
     */
    process(ctx: Context, next: any): Promise<void>;
    dispose(): void;
    private _checkTimeout;
}
