"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rowan_1 = require("rowan");
/**
 * used to intercept messages with matching message ids and terminate further processing
 **/
class Interceptor {
    _log;
    _interceptors = new Map();
    meta;
    constructor(_log) {
        this._log = _log;
    }
    /**
     * intercept any message with a matching id
     * @param id: message id to intercept
     * @param handlers: handlers or middleware
     * @returns method to remove (dispose) the interception entry.`
     */
    intercept(opt) {
        const { id, handlers } = opt;
        const log = this._log;
        if (id == undefined)
            throw Error("id cannot be undefined");
        if (id.length == 0)
            throw Error("id cannot be empty");
        if (handlers == undefined)
            throw Error("handlers cannot be undefined");
        if (handlers.length == 0)
            throw Error("handlers length cannot be zero");
        const dispose = () => {
            //console.log("disposed interceptor for", id);
            log.trace("intercept dispose " + id);
            this._interceptors.delete(id);
        };
        const middleware = handlers.map(x => rowan_1.Rowan.convertToMiddleware(x));
        if (this._interceptors.has(id)) {
            throw Error("Already intercepting messages for id: " + id);
        }
        log.trace("intercepting " + id);
        this._interceptors.set(id, {
            dispose: dispose,
            timestamp: Date.now(),
            middleware: middleware
        });
        return dispose;
    }
    /**
     * re-route processing to an interceptor's middleware
     */
    async process(ctx, next) {
        const id = ctx.in.id;
        if (this._interceptors.has(id)) {
            const interceptor = this._interceptors.get(id);
            if (interceptor) {
                return rowan_1.Rowan.process(interceptor.middleware, ctx, next);
            }
        }
        return next();
    }
    ;
    dispose() {
        for (let [key, value] of this._interceptors.entries()) {
            value.dispose();
        }
    }
    _checkTimeout() {
        let now = Date.now();
        for (let [key, value] of this._interceptors.entries()) {
            let span = now - value.timestamp;
            if (span > 5000) {
                value.dispose();
            }
        }
    }
}
exports.default = Interceptor;
