"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpgradeIncomingIterable = exports.UpgradeOutgoingIterable = exports.IteratorRouter = void 0;
const rowan_1 = require("rowan");
const request_1 = require("./request");
/**
 * Pull-Based Router for an async iterable, intended to be used by an interceptor
 **/
class IteratorRouter extends rowan_1.Rowan {
    constructor(iterable, dispose) {
        super();
        let iterator;
        let clock = setTimeout(() => {
            if (!iterator) {
                dispose();
            }
            ;
        }, 5000);
        this.use(new rowan_1.If((0, request_1.request)("NEXT"), [
            async (ctx, next) => {
                let data;
                let status;
                try {
                    let result = await iterator.next();
                    data = result.value;
                    status = result.done ? 200 : 206;
                }
                catch (err) {
                    data = err.message;
                    status = 500;
                }
                ctx.send(data != undefined ? { head: { status: status }, data: data } : { head: { status: status } });
                if (status != 206) {
                    dispose();
                }
            }
        ]));
        this.use(new rowan_1.If((0, request_1.request)("SUBSCRIBE"), [
            async (ctx, next) => {
                clearTimeout(clock);
                try {
                    if (iterator !== undefined) {
                        throw Error("Already subscribed");
                    }
                    if (iterable[Symbol.asyncIterator])
                        iterator = iterable[Symbol.asyncIterator]();
                    else
                        iterator = iterable[Symbol.iterator]();
                    ctx.send({ head: { status: 200 } });
                }
                catch (err) {
                    ctx.send({ head: { status: 500 }, data: err.message, });
                }
            }
        ]));
        this.use(new rowan_1.If((0, request_1.request)("UNSUBSCRIBE"), [
            async (ctx, next) => {
                await iterator.return();
                ctx.send({ head: { status: 200 } });
            }
        ]));
    }
}
exports.IteratorRouter = IteratorRouter;
class UpgradeOutgoingIterable {
    process(ctx, next) {
        const head = ctx.out.head;
        const data = ctx.out.data;
        if (data != undefined && data[Symbol.asyncIterator] != undefined && ((head ? head.iterable : true) || true)) {
            let iterable = data;
            let sid = ctx.connection.createId();
            let router = new IteratorRouter(iterable, function () { dispose(); });
            let dispose = ctx.connection.intercept(sid, [router]);
            head["iterable"] = sid;
        }
        return next();
    }
}
exports.UpgradeOutgoingIterable = UpgradeOutgoingIterable;
class UpgradeIncomingIterable {
    process(ctx, next) {
        if (!ctx.in || !ctx.in.data || typeof ctx.in.head["iterable"] !== "string") {
            return next();
        }
        const sid = ctx.in.head["iterable"];
        const connection = ctx.connection;
        ctx.in.data[Symbol.asyncIterator] = function () {
            let response;
            let subscribed = false;
            return {
                next: async function () {
                    if (!subscribed) {
                        subscribed = true;
                        response = await connection.request("SUBSCRIBE", undefined, undefined, { id: sid });
                        if (response.head.status != 200) {
                            throw Error(response.data);
                        }
                    }
                    response = await connection.request("NEXT", undefined, undefined, { id: sid });
                    switch (response.head.status) {
                        case 206:
                            return { value: response.data, done: false };
                        case 200:
                            return { value: undefined, done: true };
                        default:
                        case 500:
                            throw Error(response.data || "Unknown Error");
                    }
                },
                return: async function () {
                    response = await connection.request("UNSUBSCRIBE", undefined, undefined, { id: sid });
                    return { value: undefined, done: true };
                }
            };
        };
        return next();
    }
}
exports.UpgradeIncomingIterable = UpgradeIncomingIterable;
