"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Router = void 0;
const rowan_1 = require("rowan");
const path_to_regexp_1 = require("path-to-regexp");
const normalise_1 = require("../util/normalise");
;
class Router {
    /** route base path  */
    root;
    /** route descriptive name */
    name;
    /** route documentation */
    doc;
    /** route middleware */
    middleware = [];
    meta;
    _rootMatch;
    constructor(opts = {}) {
        this.root = opts.root || "/";
        this.middleware = opts.middleware || [];
        this.root = (0, normalise_1.normalisePath)(this.root);
        this.meta = {
            type: "Router",
            path: this.root,
            doc: opts.doc,
        };
        if (this.root == "") {
            this._rootMatch = () => "";
            return;
        }
        const keys = [];
        const exp = (0, path_to_regexp_1.pathToRegexp)(this.root, keys, {
            strict: false,
            end: false,
        });
        this._rootMatch = (ctx) => {
            let match = (ctx.in.head.path) ? exp.exec(ctx.in.head.path) : null;
            if (match == null) {
                return;
            }
            if (keys.length > 0) {
                ctx.params = ctx.params || {};
                for (let i = 0; i < keys.length; i += 1) {
                    ctx.params[keys[i].name] = match[i + 1];
                }
            }
            return match[0];
        };
    }
    process(ctx, next) {
        let match = this._rootMatch(ctx);
        if (match != undefined) {
            let originalPath = ctx.in.head.path;
            if (ctx.in.head.fullPath === undefined) {
                ctx.in.head.fullPath = originalPath;
            }
            ctx.in.head.path = (0, normalise_1.normalisePath)(originalPath.substr(match.length));
            return rowan_1.Rowan.process(this.middleware, ctx, function () {
                ctx.in.head.path = originalPath;
                return next();
            }).then(() => { ctx.in.head.path = originalPath; })
                .catch((err) => { ctx.in.head.path = originalPath; throw err; });
        }
        return next();
    }
    use(processor, meta) {
        this.middleware.push(rowan_1.Rowan.convertToMiddleware(processor, meta));
    }
    route(opts) {
        const path = (0, normalise_1.normalisePath)(opts.path);
        const keys = [];
        const exp = (0, path_to_regexp_1.pathToRegexp)(path, keys, {
            strict: false,
            end: (opts.end !== undefined) ? opts.end : true
        });
        const doc = opts.doc;
        const method = opts.method;
        const middleware = opts.process.map(x => rowan_1.Rowan.convertToMiddleware(x));
        const routeProcessor = {
            meta: { method, path: opts.path },
            middleware,
            process: function (ctx, next) {
                const log = ctx.connection.log;
                if (ctx.in == undefined || ctx.in.head == undefined) {
                    log.trace("in undefined, in.head undefined");
                    return next();
                }
                if (method != undefined && ctx.in.head.method !== method) {
                    log.trace(`${method} !== ${ctx.in.head.method}`);
                    return next();
                }
                log.trace(`${method} === ${ctx.in.head.method}`);
                let match = null;
                if (path == ctx.in.head.path) {
                    log.trace(`${path} == ${ctx.in.head.path}`);
                    match = [path];
                }
                else {
                    match = (ctx.in.head.path) ? exp.exec(ctx.in.head.path) : null;
                    log.trace({ match }, `available matches for ${ctx.in.head.path}`);
                    if (match == null) {
                        return next();
                    }
                    if (keys.length > 0) {
                        ctx.params = ctx.params || {};
                        for (let i = 0; i < keys.length; i += 1) {
                            ctx.params[keys[i].name] = match[i + 1];
                        }
                    }
                }
                if (match == null) {
                    log.trace("no match, nexting");
                    return next();
                }
                log.trace("matched - processing");
                let originalPath = ctx.in.head.path;
                let originalMatched = ctx.in.head.matchedPath;
                ctx.in.head.matchedPath = match[0];
                ctx.in.head.path = originalPath.substring(match[0].length);
                let restored = false;
                return rowan_1.Rowan.process(middleware, ctx, function () {
                    ctx.in.head.path = originalPath;
                    ctx.in.head.matchedPath = originalMatched;
                    restored = true;
                    return next();
                }).finally(() => {
                    if (!restored) {
                        ctx.in.head.path = originalPath;
                        ctx.in.head.matchedPath = originalMatched;
                    }
                });
            }
        };
        this.use(routeProcessor);
    }
}
exports.Router = Router;
