import { Rowan, Processor } from "rowan";
import { Wire } from "@viae/core";
import { Message, Response } from './message';
import { Context, ContextConstructor } from "./context";
import { Log } from "./log";
import { Codex } from "./util";
import { IVia, SendOptions, RequestOptions } from "./_via";
export interface RequestResponse<T = any> extends Response<T>, Disposable {
    ok: boolean;
}
/**
 * Via
 * Wraps a wire connection and processes inbound and outbound messages
 */
export declare class Via<C extends Context = Context> extends Rowan<C> implements IVia<C> {
    private _ev;
    private _active;
    private _wire;
    private _uuid;
    private _log;
    private _timeout;
    private _interceptor;
    private _encoder;
    private _before;
    private CtxCtor;
    readonly out: Rowan<Context>;
    get wire(): Wire;
    get active(): Context[];
    static Log: Log;
    constructor(opts: {
        wire: Wire;
        Ctx?: ContextConstructor;
        uuid?: () => string;
        log?: Log;
        timeout?: number;
        codex?: Codex;
    });
    get log(): Log;
    get ready(): Promise<void>;
    private _onMessage;
    /**
     * Fire and Forget - Add a message to the outgoing pipeline
     **/
    send(msg: Partial<Message>, opts?: SendOptions): Promise<void>;
    /**
     * Send a request message, add an one-off interceptor for the response. Returns promise to first response.
     * @param msg
     * @param opts
     */
    request<R, O extends RequestOptions>(method: string, path: string, data?: any, opts?: O & {
        validate?(value: any): value is R;
    }): Promise<RequestResponse & (O['accept'] extends "stream" ? {
        data: ReadableStream<R>;
    } : O['accept'] extends "object" ? {
        data: R;
    } : {})>;
    /**
     * simplified request/response  - this will deserialize the response data and return it if successful.
     * throws an error if the result status is not OK.
     **/
    /**
     * intercept an id-specific message
     * @param id message id to intercept
     * @param handlers processors to call on interception
     */
    intercept(id: string, handlers: Processor<C>[]): () => void;
    /**
     * processors to use before any  in-built (inbound)processing
     * @param processor
     */
    before(processor: Processor<C>): this;
    createId(): string;
    on(event: "close", cb: () => void): any;
    on(event: "open", cb: () => void): any;
    on(event: "error", cb: (err: Error, ctx: C) => void): any;
    off(event: "close", cb: () => void): any;
    off(event: "open", cb: () => void): any;
    off(event: "error", cb: (err: Error, ctx: C) => void): any;
}
