"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Viae = void 0;
const rowan_1 = require("rowan");
const events_1 = require("events");
const via_1 = require("./via");
const pino_1 = require("pino");
const normalise_1 = require("./util/normalise");
class Viae extends rowan_1.Rowan {
    _connections = new Array();
    _ev = new events_1.EventEmitter();
    _before = new rowan_1.Rowan();
    get connections() {
        return [...this._connections];
    }
    constructor(server, opts) {
        super((opts) ? opts.middleware : undefined);
        this.meta = { type: "Viae" };
        server.on("connection", (wire) => {
            let log = opts && opts.log ? opts.log : Viae.Log;
            let via = new via_1.Via({ wire: wire, log: log }).before(this._before).use(this);
            //if (wire.url == undefined && wire["_socket"]) {
            //  (wire as any)["url"] = wire["_socket"]["remoteAddress"]; //+ ":" + wire["_socket"]["remotePort"]
            //}
            wire.on("close", () => {
                this._connections.splice(this._connections.indexOf(via), 1);
                log.info(wire.url + " disconnected");
            });
            via.on("error", (err) => {
                log.error(wire.url + " error: " + err.message);
                wire.close();
            });
            this._connections.push(via);
            this._ev.emit("connection", via);
            log.info(wire.url + " connected");
        });
    }
    on(event, cb) {
        this._ev.on(event, cb);
    }
    before(processor) {
        this._before.use(processor);
        return this;
    }
    static Log = (0, pino_1.pino)();
    static extractRoutes(viae) {
        function getRoutes(node, root = "") {
            let routes = [];
            if (node.meta && node.meta.path !== undefined) {
                root = (0, normalise_1.normalisePath)(root, node.meta.path);
                if (node.meta.method) {
                    routes.push({ method: node.meta.method, path: root });
                }
            }
            if (node.children) {
                for (let child of node.children) {
                    routes.push(...getRoutes(child, root));
                }
            }
            return routes;
        }
        return getRoutes(rowan_1.Rowan.hierarchy(viae));
    }
}
exports.Viae = Viae;
